<?php

namespace TVn7\PrestationBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use FOS\RestBundle\Controller\FOSRestController;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\ParamConverter;
use Symfony\Component\HttpFoundation\Request;
use FOS\RestBundle\Controller\Annotations as Rest;
use FOS\RestBundle\Context\Context;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\ConflictHttpException;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

use Swagger\Annotations as SWG;
use Nelmio\ApiDocBundle\Annotation\Model;

use TVn7\PrestationBundle\Entity\Prestation;
use TVn7\PrestationBundle\Entity\DeliveryProject;

use TVn7\PrestationBundle\Form\APIDeliveryProjectType;

class APIDeliveryProjectController extends FOSRestController
{
    /**
     * Return the list of all reachable delivery project
     *
     * @Rest\Get("")
     * @Rest\View(statusCode=Response::HTTP_OK, serializerGroups = {"list", "ln_list"})
     *
     * @SWG\Response(
     *     response=Response::HTTP_OK,
     *     description="Returns the list of Delivery projects",
     *     @SWG\Schema(
     *         type="array",
     *         @SWG\Items(ref=@Model(type=DeliveryProject::class, groups={"list"}))
     *     )
     * )
     *
     * @SWG\Response(
     *     response=Response::HTTP_NOT_FOUND,
     *     description="Prestation not found"
     * )
     * @SWG\Parameter(
     *     name="prestation",
     *     in="path",
     *     type="string",
     *     description="The prestation identifier"
     * )

     * @SWG\Tag(name="DeliveryProject")
     *
     * @ParamConverter("prestation", class="TVn7PrestationBundle:prestation",   options={"mapping" : {"prestation" : "slug"}})
     */
    public function listAction(Prestation $prestation)
    {
        return $prestation->getDeliveryProjects()->filter(function($p) {
            return $this->isGranted('view', $p);
        });
    }

    /**
     * Return a delivery project
     *
     * @Rest\Get("/{project}")
     * @Rest\View(statusCode=Response::HTTP_OK, serializerGroups = {"read", "ln_read"})
     *
     * @SWG\Response(
     *     response=Response::HTTP_OK,
     *     description="Returns the delivery project",
     *     @Model(type=DeliveryProject::class, groups={"read"})
     * )
     *
     * @SWG\Response(
     *     response=Response::HTTP_NOT_FOUND,
     *     description="Delivery project requested has not been found",
     * )
     * @SWG\Tag(name="DeliveryProject")
     *
     * @SWG\Parameter(
     *     name="prestation",
     *     in="path",
     *     type="string",
     *     description="The prestation identifier"
     * )
     * @SWG\Parameter(
     *     name="project",
     *     in="path",
     *     type="string",
     *     description="The project identifier"
     * )
     *
     * @ParamConverter("prestation", class="TVn7PrestationBundle:prestation",   options={"mapping" : {"prestation" : "slug"}})
     * @ParamConverter("project", class="TVn7PrestationBundle:DeliveryProject", options={"mapping" : {"project" : "slug"}})
     */
    public function showAction(Prestation $prestation, DeliveryProject $project)
    {
        $this->checkConsistency($prestation, $project);
        $this->denyAccessUnlessGranted('view', $project);

        return $project;
    }

    /**
     * Patch the delivery project
     *
     * @Rest\Patch("/{project}")
     * @Rest\View(statusCode=Response::HTTP_OK, serializerGroups = {"read", "ln_read"})
     *
     * @SWG\Parameter(
     *     name="formData",
     *     description="Only desired attributes can be submited",
     *     required=true,
     *     in="body",
     *     schema=@SWG\Schema(
     *         ref=@Model(type=DeliveryProject::class, groups={"write"})
     *     )
     * )
     *
     * @SWG\Response(
     *     response=Response::HTTP_OK,
     *     description="Returns the delivery project updated",
     *     @Model(type=DeliveryProject::class, groups={"read"})
     * )
     *
     * @SWG\Response(
     *     response=Response::HTTP_NOT_FOUND,
     *     description="Delivery project requested has not been found",
     * )
     * @SWG\Response(
     *     response=Response::HTTP_BAD_REQUEST,
     *     description="Data invalid"
     * )
     * @SWG\Tag(name="DeliveryProject")
     *
     * @SWG\Parameter(
     *     name="prestation",
     *     in="path",
     *     type="string",
     *     description="The prestation identifier"
     * )
     * @SWG\Parameter(
     *     name="project",
     *     in="path",
     *     type="string",
     *     description="The project identifier"
     * )
     * 
     * @ParamConverter("prestation", class="TVn7PrestationBundle:prestation",   options={"mapping" : {"prestation" : "slug"}})
     * @ParamConverter("project", class="TVn7PrestationBundle:DeliveryProject", options={"mapping" : {"project" : "slug"}})
     */
    public function editAction(Request $request, Prestation $prestation, DeliveryProject $project) {
        $this->checkConsistency($prestation, $project);
        $this->denyAccessUnlessGranted('edit', $project);

        $form = $this->createForm(APIDeliveryProjectType::class, $project);

        $form->submit($request->request->all(), false);

        if ($form->isValid()) {
            $em = $this->getDoctrine()->getManager();

            $em->flush();

            return $project;
        } else {
            return $form;
        }
    }

    /**
     * Publish all tokens
     * 
     * @Rest\Put("/{project}/publish")
     * @Rest\View(statusCode=Response::HTTP_OK)
     *
     * @SWG\Response(
     *     response=Response::HTTP_OK,
     *     description="Delivery project published"
     * )
     *
     * @SWG\Response(
     *     response=Response::HTTP_NOT_FOUND,
     *     description="Delivery project not found"
     * )
     *
     * @SWG\Response(
     *     response=Response::HTTP_CONFLICT,
     *     description="The project does not contain any file or any token"
     * )
     *
     * @SWG\Response(
     *     response=Response::HTTP_FORBIDDEN,
     *     description="Publication is not allowed"
     * )
     *
     * @SWG\Tag(name="DeliveryProject")
     *
     * @SWG\Parameter(
     *     name="prestation",
     *     in="path",
     *     type="string",
     *     description="The prestation identifier"
     * )
     * @SWG\Parameter(
     *     name="project",
     *     in="path",
     *     type="string",
     *     description="The project identifier"
     * )
     *
     * @ParamConverter("prestation", class="TVn7PrestationBundle:prestation",   options={"mapping" : {"prestation" : "slug"}})
     * @ParamConverter("project", class="TVn7PrestationBundle:DeliveryProject", options={"mapping" : {"project" : "slug"}})
     */
    public function publishAction(Prestation $prestation, DeliveryProject $project)
    {
        $this->checkConsistency($prestation,$project);
        $this->denyAccessUnlessGranted('edit', $project);

        if($project->getReadyToPublish() == false) {
            throw new AccessDeniedHttpException("The project is not ready to publish.");
        }

        if($project->getFiles()->count() == 0) {
            throw new ConflictHttpException("The project does not contain any files. Unable to publish.");
        }

        if($project->getTokens()->count() == 0) {
            throw new ConflictHttpException("The project does not contain any tokens. Unable to publish.");
        }

        if($project->getPublishedAt() instanceof \DateTime) {
            throw new ConflictHttpException(sprintf("The project is already published since %s", $project->getPublishedAt()->format("c")));
        }

        $this->get('tvn7.delivery_service')->publishAll($project);
            
        return null;
    }

    public function checkConsistency(Prestation $prestation, DeliveryProject $project)
    {
        if($prestation !== $project->getPrestation()) {
            $this->createNotFoundException();
        }
    }
}
